import game
from algebra import *
import copy

class Fighter(game.Entity):
	def __init__(self, id, name="", appendId=True):
		game.Entity.__init__(self, id, name, appendId)
		self.life = 100
		self.energy = 0
		self._jump = False
		self._block = False
		self._punch = False
		self._kick = False
		#nos han dado
		self._hitted = False
		
		self.paused = True

		self._coolingTime = 0

		self.speed = Vector2d(0, 0)
		self.maxSpeed = Vector2d(150, 400)
		
		self.newPosition = copy.copy(self.position)

	def create(self, id, name="", appendId = True):
		return Fighter(id, name, appendId)

	def damage(self, damagePoints):
		self.life-= damagePoints
		self._coolingTime = 0.5
		if not self._jump:
			self.speed.x = 0

		if self.life <= 0:
			self.life = 0
			self.sounds["ko"].Play()
			self.sprite.currentAnim = "ko"
			self.sprite.anim["ko"].pos = 0
		else:
			self.sounds["ouch"].Play()
			self.sprite.currentAnim = "receive"

	def jump(self):
		if self._jump or self._coolingTime != 0 or self._block:
			return

		self._jump = True
		self.speed.y = self.maxSpeed.y
		
		self.sounds["jump"].Play()

	def punch(self):
		if self._coolingTime == 0 and not self._block:
			self._punch = True
			self._coolingTime = 0.75
			self.sprite.currentAnim = "punch"
			# si no estamos saltando cuando pegamos nos paramos
			self.sounds["punch"].Play()
			if not self._jump:
				self.speed.x = 0

	def kick(self):
		if self._coolingTime == 0 and not self._block:
			self._kick = True
			self._coolingTime = 1
			self.sprite.currentAnim = "kick"
			self.sounds["kick"].Play()
			if not self._jump:
				self.speed.x = 0

	def block(self, b):
		if self._jump or self._coolingTime != 0:
			return

		self._block = b
		
		if b:
			self.sprite.currentAnim = "block"

	def move(self, direction = 0):
		"""
		0 = parados
		1 = izquierda
		2 = derecha
		"""
		if self._coolingTime != 0:
			return

		if direction == 0:
			self.speed.x = 0
			self.sprite.currentAnim = "idle"
		elif direction == 1:
			if self._coolingTime == 0:
				self.speed.x = -self.maxSpeed.x
				self.sprite.currentAnim = "walk"
		elif direction == 2:
			if self._coolingTime == 0:
				self.speed.x = self.maxSpeed.x
				self.sprite.currentAnim = "walk"

		if self._jump:
			self.sprite.currentAnim = "jump"

	def update(self, time):
		game.Entity.update(self, time)

		if self._block:
			return

		self.newPosition.x = self.position.x + self.speed.x * time

		if self._jump:
			self.newPosition.y = self.position.y + self.speed.y * time
			self.speed.y-= 700 * time
			if self.newPosition.y <= 0:
				self.newPosition.y = 0
				self._jump = False
				if self._coolingTime > 0:
					self._coolingTime = 0
				if self.life == 0 or self.enemy.life == 0:
					self.speed.x = 0
		else:
			if self.life == 0 or self.enemy.life == 0:
				self.speed.x = 0


		if self._coolingTime > 0:
			self._coolingTime-= time
			if self._coolingTime < 0:
				self._coolingTime = 0

		if self._punch:
			self._punch = False
		if self._kick:
			self._kick = False
