from algebra import *
from gfx import *
from gui import *
from audio import *
import copy

import pygame, pygame.image, pygame.key, pygame.mixer
from pygame.locals import *

class Config:
	def __init__(self):
		self.gfxConfig = gfx.Config()
		#self.audioConfig = audioConfig()
		#...

	@classmethod
	def load(self, filename):
		return pickle.load(open(filename, "r"))

	def save(self, filename):
		pickle.dump(self, open(filename, "w"))

"""
Sera interesante tener un diccionario de string->boolean y desde
un fichero externo asociar nombres a dispositivos de entrada tipo:

"jump" keyboard space
"shot" keyboard r_ctrl, mouse left_button

y posiblemente tambin un script en python que ejecutase la orden,
aunque esto puede ser un agujero de seguridad

"""

translateKeycode = {
	'up': K_UP, 'down':K_DOWN, 'left':K_LEFT, 'right':K_RIGHT,
	'space':K_SPACE, 'r_ctrl':K_RCTRL, 'l_ctrl':K_LCTRL, 'tab':K_TAB,
	'l_shift': K_LSHIFT, 'r_shift': K_RSHIFT
	}

class Input:
	def __init__(self, configFile = None):
		self.action = dict()
		self.key = dict()
		self.joybutton = dict()
		self.joyaxis = dict()

		pygame.joystick.init()
		if pygame.joystick.get_count() > 0:
			print "se ha encontrado un joystick"
			self.joystick = pygame.joystick.Joystick(0)
			self.joystick.init()
			print self.joystick.get_name()

		if configFile:
			self.load(configFile)

	def load(self, filename):
		f = open(filename, "rt")
		for w in f:
			if w != "endoffile":
				s = w.split()
				if len(s) >= 3:
					if s[1] == "key":
						try:
							keycode = translateKeycode[s[2]]
						except KeyError:
							keycode = ord(s[2][0])
						self.addAction(s[0], key = keycode)
					elif s[1] == "joybutton":
						self.addAction(s[0], joybutton = int(s[2]))
					elif s[1] == "joyaxis":
						self.addAction(s[0], joyaxis = int(s[2]), joyaxisSig = int(s[3]))
					elif s[1] == "mouse":
						print "[ERROR] entrada por ratn no soportada"

		f.close()

	def addAction(self, actionName, key  = None, joybutton = None, mousebutton = None,
		joyaxis = None, joyaxisSig = None):
		self.action[actionName] = False
		if key != None:
			self.key[key] = actionName
		if joybutton != None:
			self.joybutton[joybutton] = actionName
		if joyaxis != None:
			self.joyaxis[(joyaxis, joyaxisSig)] = actionName

	def checkAction(self, actionName):
		"""
		comprobamos el estado de la accin 'actionName'
		en caso de no existir, devolveramos False
		"""
		try:
			return self.action[actionName]
		except KeyError:
			return False

	def update(self, event = None):
		if event == None:
			return

		if event.type == KEYDOWN:
			for key, actionName in self.key.iteritems():
				if event.key == key:
					self.action[actionName] = True
		elif event.type == KEYUP:
			for key, actionName in self.key.iteritems():
				if event.key == key:
					self.action[actionName] = False
		elif event.type == JOYAXISMOTION:
			sig = 0
			if event.value > 0.05:
				sig = 1
			elif event.value < -0.05:
				sig = -1
			else:
				if self.joyaxis.has_key((event.axis, 1)):
					self.action[self.joyaxis[(event.axis, 1)]] = False
				if self.joyaxis.has_key((event.axis, -1)):
					self.action[self.joyaxis[(event.axis, -1)]] = False
				
			if self.joyaxis.has_key((event.axis, sig)):
				self.action[self.joyaxis[(event.axis, sig)]] = True
		elif event.type == JOYBUTTONDOWN:
			if self.joybutton.has_key(event.button):
				self.action[self.joybutton[event.button]] = True
		elif event.type == JOYBUTTONUP:
			if self.joybutton.has_key(event.button):
				self.action[self.joybutton[event.button]] = False
		#else:
		#	print event.type


class Entity(Positionable):
	def __init__(self, id, name = "", appendId = True):
		Positionable.__init__(self)
		self.id = id
		if name == "":
			self.name = "entity" + str(self.id)
		else:
			self.name = name
			if appendId:
				self.name+= str(self.id)

		self.sprite = None
		self.animationState = AnimationState

		self.speed = Vector2d(0, 0)

	def update(self, time):
		pass

	def create(self, id, name="", appendId = True):
		return Entity(id, name, appendId)

	def __str__(self):
		return "id = " + str(self.id) + ", name = " + self.name

class World:
	def __init__(self):
		self.entity = dict()
		self.maxId = 0
		self.entityCatalog = None

	def createEntity(self, name = "", appendId = True, type = None):
		if self.entityCatalog != None and type != None:
			e = self.entityCatalog[type].create(self.maxId, name, appendId)
		else:
			e = Entity(self.maxId, name, appendId)
		self.maxId+= 1
		self.entity[e.name] = e
		return e

	def addEntity(self, entity):
		pass

	"""
	hace un update del mundo de 'time' segundos
	"""
	def update(self, time):
		for name, e in self.entity.iteritems():
			e.update(time)

class Game:
	def __init__(self, inputFile = None):
		self.initVideo()

		self.gfxEngine = GFXEngine()
		#self.soundManager = SoundManager()
		self.gui = GUI()

		self.input = Input(inputFile)

		self.world = World()

		self.quit = False
		self.frame = 0
		self.pause = False

		self.updateFunctions = []

		self.player = self.world.createEntity("player", False)

		self.entityCatalog = dict()
		self.world.entityCatalog = self.entityCatalog

	def initVideo(self):
		pass

	def run(self):
		self.quit = False
		t = t0 = pygame.time.get_ticks()
		while not self.quit:
			self.input.update()
			for event in pygame.event.get():
				self.input.update(event)

				if event.type == QUIT:
					self.quit = True
				if event.type == KEYDOWN:
					if event.key == K_ESCAPE:
						self.quit = True
					else:
						self.gui.keyPressed(event.key)
				if event.type == MOUSEBUTTONDOWN:
					self.gui.click(event.pos[0], event.pos[1])

			self.update((t - t0)*0.001)
			self.render()
			t0 = t
			t = pygame.time.get_ticks()

		pygame.time.delay(1000)

	def createEntity(self, type = None, position = Vector2d(0, 0)):
		e = self.world.createEntity(type = type)
		e.position = copy.copy(position)

		sprite = Sprite(e.position)
		#sprite.addFrame("data/bomba1.png")
		e.sprite = sprite
		self.gfxEngine.root.addChild(sprite)

	def registerEntity(self, name, instance):
		self.entityCatalog[name] = instance

	# hay que distinguir entre cosas que se pausan y cosas que no
	def update(self, time):
		for u in self.updateFunctions:
			u(time)

		if self.input.checkAction("up"):
			self.player.sprite.position.y+= time*50
		if self.input.checkAction("down"):
			self.player.sprite.position.y-= time*50
		if self.input.checkAction("left"):
			self.player.sprite.position.x-= time*50
		if self.input.checkAction("right"):
			self.player.sprite.position.x+= time*50

		if self.input.checkAction("shot"):
			self.createEntity(self.player.sprite.position)

		if self.input.checkAction("cameraLeft"):
			self.gfxEngine.camera.position.x-= time*150
		if self.input.checkAction("cameraRight"):
			self.gfxEngine.camera.position.x+= time*150
		if self.input.checkAction("cameraUp"):
			self.gfxEngine.camera.position.y+= time*150
		if self.input.checkAction("cameraDown"):
			self.gfxEngine.camera.position.y-= time*150

		if self.input.checkAction("cameraZoomIn"):
			self.gfxEngine.camera.zoom(1+time)
		if self.input.checkAction("cameraZoomOut"):
			self.gfxEngine.camera.zoom(1-time)

		self.gfxEngine.update(time)

		#self.world.update(time)

	def render(self):
		glClear(GL_COLOR_BUFFER_BIT)

		self.gfxEngine.render()
		self.gui.render()

		self.frame+= 1

		pygame.display.flip()
