from OpenGL.GL import *
from OpenGL.GLU import *
from algebra import *
from PIL import Image
import pickle
import pygame
import copy

class AnimationState:
	def __init__(self):
		self.speed = 1
		self.pos = 0
		self.frame = []
		self.center = []
		self.frames = 0
		self.fps = 4
		self.currentFrame = 0
		self.loop = True

	def update(self, time):
		self.pos+= self.speed * time

class Color:
	def __init__(self, r = 1, g = 1, b = 1, a = 1):
		self.r = r
		self.g = g
		self.b = b
		self.a = a

	def render(self):
		glColor4f(self.r, self.g, self.b, self.a)

class Texture:
	def __init__(self):
		self.size = Vector2d(0, 0)

	def render(self):
		glEnable(GL_TEXTURE_2D)
		glBindTexture(GL_TEXTURE_2D, self.id)

class TextureManager:
	def __init__(self):
		self.texture = dict()
		self.defaultTexture = Texture()
		self.defaultTexture.id = 0

	def load(self, filename):
		try:
			img = Image.open(filename)
		except IOError:
			print "[ERROR] TextureManager.load()" + filename
			return self.defaultTexture
		#print img.mode
		tex = Texture()
		tex.id = glGenTextures(1)
		tex.size.x = img.size[0]
		tex.size.y = img.size[1]

		glBindTexture(GL_TEXTURE_2D, tex.id)
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR)
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR)
		
		if img.mode == "RGBA":
			format = GL_RGBA
		elif img.mode == "RGB":
			format = GL_RGB
		else:
			print "ERROR: formato no soportado"

		#str = img.tostring()
		#glTexImage2D( GL_TEXTURE_2D, 0, GL_RGBA, img.size[0], img.size[1], 0,GL_RGBA, GL_UNSIGNED_BYTE, str )
		gluBuild2DMipmaps(GL_TEXTURE_2D, GL_RGBA, img.size[0], img.size[1], format, GL_UNSIGNED_BYTE, img.tostring())
		return tex

textureManager = TextureManager()

"""

animationName fps files ...

idle 5 idle0.png idle2.png idle3.png idle4.png idle5.png
walk 15 walk0.png walk1.png walk2.png walk3.png

o un xml to guapo

<sprite name="malo">
	<animations>
		<animation name="idle" fps="5">
			<frames>
				<frame id="0">data/idle0.png</frame>
				...
			</frames>
		</animation>
	</animations>
</sprite>

o en

<entity name="malo">

	<hitpoints value="100"/>

	<script value="data/scripts/malo.py"/>

	<animations/>

	<etc/>

</entity>

"""

class Camera:
	def __init__(self):
		self.position = Vector2d(0, 0)
		self.size = Vector2d(640, 480)

	def render(self):
		glMatrixMode(GL_PROJECTION)
		glLoadIdentity()
		gluOrtho2D(0, self.size.x, 0, self.size.y)
		glMatrixMode(GL_MODELVIEW)
		glLoadIdentity()
		glTranslatef(-self.position.x + self.size.x/2, -self.position.y + self.size.y/2, 0)

	def zoom(self, value):
		self.size.x/= value
		self.size.y/= value

class Positionable:
	def __init__(self, position = Vector2d(0, 0)):
		self.position = copy.copy(position)

class Node(Positionable):
	def __init__(self, position = Vector2d(0, 0)):
		Positionable.__init__(self, position)
		self.name = ""
		self.bbox = None
		self.culled = False

	def setBBox(self, bbox):
		self.bbox = bbox

	def calculateBBox(self):
		pass

	def traverse(self):
		pass

	def update(self, time):
		pass

class Group(Node):
	def __init__(self, position = Vector2d(0, 0)):
		Node.__init__(self, position)
		self.child = []

	def addChild(self, child):
		self.child.append(child)

	def render(self):
		if self.culled:
			return

		glPushMatrix()
		glTranslatef(self.position.x, self.position.y, 0)
		for c in self.child:
			c.render()
		glPopMatrix()

	def calculateBBox(self):
		if self.bbox == None:
			self.bbox = BBox()

		self.bbox.min = Vector(1e100, 1e100)
		self.bbox.max = Vector(-1e100, -1e100)

		for c in self.child:
			c.calculateBBox()

			if c.bbox.min.x < self.bbox.min.x:
				self.bbox.min.x = c.bbox.min.x
			if c.bbox.min.y < self.bbox.min.y:
				self.bbox.min.y = c.bbox.min.y
			if c.bbox.max.x < self.bbox.max.x:
				self.bbox.max.x = c.bbox.max.x
			if c.bbox.max.y < self.bbox.max.y:
				self.bbox.max.y = c.bbox.max.y

	def traverse(self):
		for c in self.child:
			c.traverse()

	def update(self, time):
		for c in self.child:
			c.update(time)

class Sprite(Node):
	def __init__(self, position = Vector2d(0, 0)):
		Node.__init__(self, position)
		self.animation = dict()
		self.size = Vector2d(20, 20)
		self.bbox = BBox(self.position, self.position + self.size)
		self.flip = False
		self.anim = dict()
		self.currentAnim = "idle"
		self.color = Color(1, 1, 1)

	def render(self):
		if self.culled:
			return

		glPushMatrix()
		glTranslatef(self.position.x, self.position.y, 0)

		self.anim[self.currentAnim].frame[self.anim[self.currentAnim].currentFrame].render()
		size = self.anim[self.currentAnim].frame[self.anim[self.currentAnim].currentFrame].size

		center = self.anim[self.currentAnim].center[self.anim[self.currentAnim].currentFrame]
		glTranslatef(center.x, center.y, 0)

		#self.currentFrame+= 1
		
		if self.flip:
			min = 1
			max = 0
		else:
			min = 0
			max = 1

		self.color.render()
		glBegin(GL_QUADS)
		glTexCoord2f(min, 1)
		glVertex2f(-size.x/2, 0)
		glTexCoord2f(max, 1)
		glVertex2f(size.x/2, 0)
		glTexCoord2f(max, 0)
		glVertex2f(size.x/2, size.y)
		glTexCoord2f(min, 0)
		glVertex2f(-size.x/2, size.y)
		glEnd()

		glPopMatrix()

	def addFrame(self, anim, filename, center = Vector2d(0, 0)):
		tex = textureManager.load(filename)

		if not self.anim.has_key(anim):
			self.anim[anim] = AnimationState()

		self.anim[anim].frame.append(tex)
		self.anim[anim].center.append(center)
		self.anim[anim].frames+= 1

	def calculateBBox(self):
		self.bbox.min = self.position
		self.bbox.max = self.position + self.size

	def traverse(self):
		self.render()

	def update(self, time):
		self.anim[self.currentAnim].pos+=time * self.anim[self.currentAnim].speed;
		self.anim[self.currentAnim].currentFrame = int(self.anim[self.currentAnim].pos*self.anim[self.currentAnim].fps)
		if self.anim[self.currentAnim].loop:
			self.anim[self.currentAnim].currentFrame%= self.anim[self.currentAnim].frames
		else:
			if self.anim[self.currentAnim].currentFrame >= self.anim[self.currentAnim].frames:
				self.anim[self.currentAnim].currentFrame = self.anim[self.currentAnim].frames - 1
class Config:
	def __init__(self):
		self.resolution = (640, 480)
		self.depth = 32
		self.antialias = False
		self.fullscreen = False

	@classmethod
	def load(self, filename):
		return pickle.load(open(filename, "r"))

	def save(self, filename):
		pickle.dump(self, open(filename, "w"))

class GFXEngine:
	def __init__(self):
		self.camera = Camera()
		self.root = Group()
		self.configuration = Config()
		glEnable(GL_BLEND)
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)

	def init(self):
		pass

	def update(self, time):
		self.root.update(time)

	def render(self):
		self.camera.render()

		self.root.traverse()

	def project(self, coords):
		v = Vector2d(0, 0)
		print "error: GFXEngine.project() - no me han implementado"
		return v

def surface2texture(s, tex):
	if tex == None:
		tex = Texture()
		tex.id = glGenTextures(1)

	glBindTexture(GL_TEXTURE_2D, tex.id)
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR)
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR)

	#if img.mode == "RGBA":
	#	format = GL_RGBA
	#elif img.mode == "RGB":
	#	format = GL_RGB
	#else:
	#	print "ERROR: formato no soportado"
	format = GL_RGBA

	#str = img.tostring()
	#glTexImage2D( GL_TEXTURE_2D, 0, GL_RGBA, img.size[0], img.size[1], 0,GL_RGBA, GL_UNSIGNED_BYTE, str )
	data = pygame.image.tostring(s, "RGBA", 1)

	gluBuild2DMipmaps(GL_TEXTURE_2D, GL_RGBA, s.get_width(), s.get_height(), format, GL_UNSIGNED_BYTE, data )

	return tex
	
