import gfx
import pygame
from OpenGL.GL import *
from OpenGL.GLU import *
from algebra import *
from pygame.locals import *
import math

class Widget:
	def __init__(self):
		self.onClick = None
		self.onKeyPressed = None
		self.position = Vector2d(0, 0)
		self.size = Vector2d(0, 0)

	def checkClick(self, x, y):
		if x < self.position.x:
			return False
		if y < self.position.y:
			return False
		if x > self.position.x + self.size.x:
			return False
		if y > self.position.y + self.size.y:
			return False

		return True

	def click(self, x, y):
		if not self.checkClick(x, y):
			return None

		if self.onClick != None:
			return self.onClick()

	def keyPressed(self, key):
		if self.onKeyPressed != None:
			self.onKeyPressed(key)

	def render(self):
		print "Widget.render implementame!"

class Container(Widget):
	def __init__(self):
		Widget.__init__(self)
		self.widget = []

	def addWidget(self, widget):
		self.widget.append(widget)

	def click(self, x, y):
		if not self.checkClick(x, y):
			return None
		
		if self.onClick != None:
			return self.onClick()

		for w in self.widget:
			w.click(x, y)

	def keyPressed(self, key):
		if self.onKeyPressed != None:
			self.onKeyPressed(key)

		for w in self.widget:
			w.keyPressed(key)

	def render(self):
		for w in self.widget:
			w.render()

class Image(Widget):
	def __init__(self, image = None):
		Widget.__init__(self)
		if image:
			self.tex = gfx.textureManager.load(image)
		else:
			self.tex = None

		self.angle = 0.0

	def setImage(self, image):
		self.tex = gfx.textureManager.load(image)

	def render(self):
		if self.tex:
			self.tex.render()

			glPushMatrix();

			glTranslatef(self.position.x, self.position.y, 0)

			if(self.angle != 0.0):
				glTranslatef(self.size.x / 2, self.size.y / 2, 0)
				glRotatef(self.angle, 0, 0, 1)
				glTranslatef(-self.size.x / 2, -self.size.y / 2, 0)

			glBegin(GL_QUADS)
			glTexCoord2f(0, 0)
			glVertex2f(0, 0)
			glTexCoord2f(1, 0)
			glVertex2f(self.size.x, 0)
			glTexCoord2f(1, 1)
			glVertex2f(self.size.x, self.size.y)
			glTexCoord2f(0, 1)
			glVertex2f(0, self.size.y)
			glEnd()

			glPopMatrix()


class Button(Widget):
	def __init__(self):
		Widget.__init__(self)
		self.size = Vector2d(100, 100)
		self.position = Vector2d(0, 0)
		self.color = gfx.Color(0.75, 0.75, 0.75, 1)
		self.backGround = None

	def render(self):
		self.color.render()

		if self.backGround != None:
			self.backGround.render()
		else:
			glDisable(GL_TEXTURE_2D)

		glBegin(GL_QUADS)
		glTexCoord2f(0, 0)
		glVertex2f(self.position.x, self.position.y)
		glTexCoord2f(1, 0)
		glVertex2f(self.position.x + self.size.x, self.position.y)
		glTexCoord2f(1, 1)
		glVertex2f(self.position.x + self.size.x, self.position.y + self.size.y)
		glTexCoord2f(0, 1)
		glVertex2f(self.position.x, self.position.y + self.size.y)
		glEnd()

class CheckBox(Widget):
	def __init__(self, backgroundImage = None, checkedImage = None):
		Widget.__init__(self)
		self.checked = False
		self.style = 0
		self.onClick = self._onClick
		
		if backgroundImage:
			self.backgroundImage = gfx.textureManager.load(backgroundImage)
		else:
			self.backgroundImage = backgroundImage

		if checkedImage:
			self.checkedImage = gfx.textureManager.load(checkedImage)
		else:
			self.checkedImage = checkedImage

	def _onClick(self):
		self.checked = not self.checked

	def setBackgroundImage(self, backgroundImage):
		if backgroundImage:
			self.backgroundImage = gfx.textureManager.load(backgroundImage)
		else:
			self.backgroundImage = backgroundImage

	def setCheckedImage(self, checkedImage):
		if checkedImage:
			self.checkedImage = gfx.textureManager.load(checkedImage)
		else:
			self.checkedImage = checkedImage


	def render(self):
		glPushMatrix();
		glTranslatef(self.position.x, self.position.y, 0)

		glLineWidth(4)
		
		if self.backgroundImage:
			glColor3f(1, 1, 1)
			self.backgroundImage.render()
			glBegin(GL_QUADS)
			glTexCoord2f(0, 0)
			glVertex2f(0, 0)
			glTexCoord2f(1, 0)
			glVertex2f(self.size.x, 0)
			glTexCoord2f(1, 1)
			glVertex2f(self.size.x, self.size.y)
			glTexCoord2f(0, 1)
			glVertex2f(0, self.size.y)
			glEnd()
		else:
			glDisable(GL_TEXTURE_2D)
			glColor3f(1, 1, 1)
			glBegin(GL_QUADS);
			glVertex2f(0, 0);
			glVertex2f(self.size.x, 0);
			glVertex2f(self.size.x, self.size.y);
			glVertex2f(0, self.size.y);
			glEnd()
			
	
			glColor3f(0, 0, 0)
			glBegin(GL_LINE_LOOP);
			glVertex2f(0, 0);
			glVertex2f(self.size.x, 0);
			glVertex2f(self.size.x, self.size.y);
			glVertex2f(0, self.size.y);
			glEnd()

		if(self.checked):
			if self.checkedImage:
				glColor3f(1, 1, 1)
				self.checkedImage.render()
				glBegin(GL_QUADS)
				glTexCoord2f(0, 0)
				glVertex2f(0, 0)
				glTexCoord2f(1, 0)
				glVertex2f(self.size.x, 0)
				glTexCoord2f(1, 1)
				glVertex2f(self.size.x, self.size.y)
				glTexCoord2f(0, 1)
				glVertex2f(0, self.size.y)
				glEnd()
			else:
				glColor3f(0, 0, 0)
				glBegin(GL_LINES);
				glVertex2f(0, 0);
				glVertex2f(self.size.x, self.size.y);
				glVertex2f(self.size.x, 0);
				glVertex2f(0, self.size.y);
				glEnd()

		glPopMatrix();

class ProgressBar(Widget):
	def __init__(self):
		Widget.__init__(self)
		self.value = 0
		self.showText = False
		self.min = 0
		self.max = 100
		self.style = None # solid, gradient, ...
		self.color = gfx.Color(0, 0, 1)

	def render(self):
		x = self.size.x * self.value / self.max
		if x < 0:
			x = 0
		if x > self.size.x:
			x = self.size.x

		self.color.render()
		glDisable(GL_TEXTURE_2D)
		glBegin(GL_QUADS)
		glVertex2f(self.position.x, self.position.y)
		glVertex2f(self.position.x + x, self.position.y)
		glVertex2f(self.position.x + x, self.position.y + self.size.y)
		glVertex2f(self.position.x, self.position.y + self.size.y)
		glEnd()

class Font:
	def __init__(self, fontname, size):
		self.fontname = fontname
		self.size = size

		try: # hacerlo slo una vez
			self.font = pygame.font.Font(fontname, size)
		except IOError:
			print "[ERROR] no se ha encontrado fuente: " + self.fontname

"""
de momento utilizamos pygame.font, y pygame.surfarray, pero este ltimo
depende del mdulo Numeric que por defecto no viene con el wpython
la PIL tambin tiene soporte para fuentes, igual sera interesante utilizarlo
"""
class Label(Widget):
	def __init__(self, text = "", color = gfx.Color(1, 1, 1), fontname = None, size = 16, font = None):
		Widget.__init__(self)
		self.color = color
		self.text = text
		self.tex = None
		if fontname:
			self.font = Font(fontname, size)
		else:
			self.fontname = pygame.font.get_default_font()

		if font:
			self.font = font

		self.calculateTexture()

	def setText(self, text):
		self.text = text
		self.calculateTexture()

	def calculateTexture(self):
		s = self.font.font.render(self.text, True, (self.color.r*255, self.color.g*255, self.color.b*255))
		# fixme: hay que reutilizar el id de textura!
		self.tex = gfx.surface2texture(s, self.tex)
		self.size.x = s.get_width()
		self.size.y = s.get_height()

	def render(self):
		glPushMatrix()
		glTranslatef(self.position.x, self.position.y, 0)
		
		self.color.render()

		if self.tex:
			self.tex.render()
			glBegin(GL_QUADS)
			glTexCoord2f(0, 1)
			glVertex2f(0, 0)
			glTexCoord2f(1, 1)
			glVertex2f(self.size.x, 0)
			glTexCoord2f(1, 0)
			glVertex2f(self.size.x, self.size.y)
			glTexCoord2f(0, 0)
			glVertex2f(0, self.size.y)
			glEnd()
		#pos = 0

		#for c in self.text:
		#	glBegin(GL_QUADS)
		#	glVertex2f(pos, 0)
		#	glVertex2f(pos + 12, 0)
		#	glVertex2f(pos + 12, 12)
		#	glVertex2f(pos, 12)
		#	glEnd()
		#	pos+=12

		glPopMatrix()

class InputText(Label):
	def __init__(self, text = "", color = gfx.Color(1, 1, 1), fontname = None, size = 16, font = None):
		Label.__init__(self, text, color, fontname, size, font)
		self.onKeyPressed = self._keyPressed

	def _keyPressed(self, key):
		if key > 255:
			return

		if key == K_BACKSPACE:
			self.text = self.text[0:len(self.text)-1]
		else:
			self.text+=  chr(key)
		self.calculateTexture()

	def render(self):
		Label.render(self)

"""
MenuAction: lo que se espera q haga el Menuitem
"""
class MenuAction:
    def perform(self):
        #poner aqui lo q tiene q hacer
        pass
"""
Elemento del menu (es una Label con unos efectos anyadidos en fase de rendering... sorry pero no necesitaba un Positionable)
cuando esta selecionado cambia color y bota.
Pa inicializarlo necesita una MenuAction, el texto q va a aparecer y la posicion
"""
class Menuitem(Label):
    def __init__(self, action, text, position):
        Label.__init__(self, text, gfx.Color(1, 0.5, 0.5), font=Font("data/comic.ttf", 32))
        self.highlighted=False
        self.action=action
        self.position=position
        self.boingboing=0.0
        self.origy=self.position.y
        
    def trigger(self):
        self.action.perform()
        self._trigger()
        
    def render(self):
        if self.highlighted==False:
            self.position.y=self.origy
            Label.render(self)
        else:
            #effetto speciale (rimbalzo?)
            self.boingboing=self.boingboing+0.05
            if self.boingboing > 180: self.boingboing=0
            
            glPushMatrix()
            self.position.y=self.origy-2*math.sin(self.boingboing)
            glTranslatef(self.position.x, self.position.y, 0)
            #print "rendering highlighted item at y:"+str(self.position.y)
            
            self.color.render()
            
            if self.tex:
                self.tex.render()
                glBegin(GL_QUADS)
                glColor3f(1.0,1.0,1.0)
                glTexCoord2f(0, 1)
                glVertex2f(0, 0)
                glTexCoord2f(1, 1)
                glColor3f(1.0,0.5,1.0)            
                glVertex2f(self.size.x, 0)
                glTexCoord2f(1, 0)
                glColor3f(1.0,1.0,1.0)
                glVertex2f(self.size.x, self.size.y)
                glTexCoord2f(0, 0)
                glColor3f(0.5,0.5,1.0)
                glVertex2f(0, self.size.y)
                glEnd()
            glPopMatrix()

class Menu(Container):
	def __init__(self):
		Container.__init__(self)
		self.selected = 0

		self.onKeyPressed = self._onKeyPressed

	def _onKeyPressed(self, key):
		if key == K_DOWN:
			self.selected+= 1
			self.selected%= len(self.widget)
		elif key == K_UP:
			self.selected-= 1
			self.selected%= len(self.widget)
		elif key == K_SPACE:
			self.widget[self.selected].trigger()

		for w in self.widget:
			w.color = gfx.Color(1.0, 0.3, 0.5)
		self.widget[self.selected].color = gfx.Color(0.3, 0.6, 1.0)

class GUI(Container):
	def __init__(self):
		Container.__init__(self)
		self.mousePos = Vector2d(0, 0)
		self.mouseSprite = None
		self.showCursor = True
		self.position = Vector2d(0, 0)
		self.size = Vector2d(640, 480)
		self.focusWidget = None

	def render(self):
		glMatrixMode(GL_PROJECTION)
		glLoadIdentity()
		gluOrtho2D(0, 640, 480, 0)
		glMatrixMode(GL_MODELVIEW)
		glLoadIdentity()

		glEnable(GL_BLEND)
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)

		for w in self.widget:
			w.render()

		self.renderMouseCursor()

	def renderMouseCursor(self):
		if self.showCursor and self.mouseSprite != None:
			self.mouseSprite.render()

def a():
	print "'a' Called"
	return 123


if __name__ == "__main__":
	w = Widget()
	w.onClick = a
	if w.click(10, 10) != 123:
		print "Error 'onClick'"

